@@ -75,12 +75,6 @@ in this Software without prior written authorization from The Open Group.
 #if defined(SVR4)
 # include    <sys/sockio.h>
 #endif
-#if defined(SVR4) && defined(PowerMAX_OS)
-# include    <sys/stropts.h>
-#endif
-#if defined(SYSV) && defined(i386)
-# include    <sys/stream.h>
-#endif
 
 #include    "dm_socket.h"
 
@@ -107,22 +101,14 @@ in this Software without prior written authorization from The Open Group.
 # define ishexdigit(c)	(isdigit(c) || ('a' <= (c) && (c) <= 'f'))
 #endif
 
-#ifdef hpux
-# include <sys/utsname.h>
-# ifdef HAS_IFREQ
-#  include <net/if.h>
-# endif
-#else
-# ifdef __convex__
-#  include <sync/queue.h>
-#  include <sync/sema.h>
-# endif
-# include <net/if.h>
-#endif /* hpux */
-
+#include    <net/if.h>
 #include    <netdb.h>
 #include    <X11/keysym.h>
 
+#if defined(IPv6) && !defined(AF_INET6)
+#error "Cannot build IPv6 support without AF_INET6"
+#endif
+
 static int FromHex (char *s, char *d, int len);
 static int oldline;
 
@@ -181,7 +167,7 @@ typedef struct _hostName {
 static HostName    *hostNamedb;
 
 static int  socketFD;
-#if defined(IPv6) && defined(AF_INET6)
+#ifdef IPv6
 static int  socket6FD;
 #endif
 
@@ -193,39 +179,6 @@ static int  pingTry;
 static XdmcpBuffer	directBuffer, broadcastBuffer;
 static XdmcpBuffer	buffer;
 
-#if (defined(SVR4) && !defined(sun) && !defined(__sgi) && !defined(NCR)) && defined(SIOCGIFCONF)
-
-/* Deal with different SIOCGIFCONF ioctl semantics on these OSs */
-
-static int
-ifioctl (int fd, int cmd, char *arg)
-{
-    struct strioctl ioc;
-    int ret;
-
-    bzero((char *) &ioc, sizeof(ioc));
-    ioc.ic_cmd = cmd;
-    ioc.ic_timout = 0;
-    if (cmd == SIOCGIFCONF)
-    {
-	ioc.ic_len = ((struct ifconf *) arg)->ifc_len;
-	ioc.ic_dp = ((struct ifconf *) arg)->ifc_buf;
-    }
-    else
-    {
-	ioc.ic_len = sizeof(struct ifreq);
-	ioc.ic_dp = arg;
-    }
-    ret = ioctl(fd, I_STR, (char *) &ioc);
-    if (ret >= 0 && cmd == SIOCGIFCONF)
-	((struct ifconf *) arg)->ifc_len = ioc.ic_len;
-    return(ret);
-}
-#else /* (SVR4 && !sun && !NCR) && SIOCGIFCONF */
-# define ifioctl ioctl
-#endif /* (SVR4 && !sun && !NCR) && SIOCGIFCONF */
-
-
 /* ARGSUSED */
 static void
 PingHosts (XtPointer closure, XtIntervalId *id)
@@ -235,7 +188,7 @@ PingHosts (XtPointer closure, XtIntervalId *id)
 
     for (hosts = hostAddrdb; hosts; hosts = hosts->next)
     {
-#if defined(IPv6) && defined(AF_INET6)
+#ifdef IPv6
 	if ( ((struct sockaddr *) hosts->addr)->sa_family == AF_INET6 )
 	    sfd = socket6FD;
 	else
@@ -277,7 +230,7 @@ RebuildTable (int size)
 	    newTable[i] = names->fullname;
 	qsort (newTable, size, sizeof (char *), HostnameCompare);
     }
-    XawListChange (list, newTable, size, 0, TRUE);
+    XawListChange (list, (_Xconst char **) newTable, size, 0, TRUE);
     free (NameTable);
     NameTable = newTable;
     NameTableSize = size;
@@ -298,7 +251,7 @@ AddHostname (ARRAY8Ptr hostname, ARRAY8Ptr status, struct sockaddr *addr, int wi
 	hostAddr.length = 4;
 	connectionType = FamilyInternet;
 	break;
-#if defined(IPv6) && defined(AF_INET6)
+#ifdef IPv6
     case AF_INET6:
 	hostAddr.data = (CARD8 *) &((struct sockaddr_in6 *) addr)->sin6_addr;
 	hostAddr.length = 16;
@@ -334,7 +287,7 @@ AddHostname (ARRAY8Ptr hostname, ARRAY8Ptr status, struct sockaddr *addr, int wi
 	    switch (addr->sa_family)
 	    {
 	    case AF_INET:
-#if defined(IPv6) && defined(AF_INET6)
+#ifdef IPv6
 	    case AF_INET6:
 #endif
 		{
@@ -429,7 +382,7 @@ ReceivePacket (XtPointer closure, int *source, XtInputId *id)
     ARRAY8	    hostname = {0, NULL};
     ARRAY8	    status = {0, NULL};
     int		    saveHostname = 0;
-#if defined(IPv6) && defined(AF_INET6)
+#ifdef HAVE_STRUCT_SOCKADDR_STORAGE
     struct sockaddr_storage addr;
 #else
     struct sockaddr addr;
@@ -525,9 +478,6 @@ RegisterHostaddr (struct sockaddr *addr, int len, xdmOpCode type)
 static void
 RegisterHostname (char *name)
 {
-#if !defined(IPv6) || !defined(AF_INET6)
-    struct hostent	*hostent;
-#endif
     struct sockaddr_in	in_addr;
     struct ifconf	ifc;
     register struct ifreq *ifr;
@@ -538,7 +488,7 @@ RegisterHostname (char *name)
     {
 	ifc.ifc_len = sizeof (buf);
 	ifc.ifc_buf = buf;
-	if (ifioctl (socketFD, (int) SIOCGIFCONF, (char *) &ifc) < 0)
+	if (ioctl (socketFD, (int) SIOCGIFCONF, (char *) &ifc) < 0)
 	    return;
 
 	cplim = (char *) ifc.ifc_req + ifc.ifc_len;
@@ -557,13 +507,13 @@ RegisterHostname (char *name)
 		struct ifreq    broad_req;
 
 		broad_req = *ifr;
-		if (ifioctl (socketFD, SIOCGIFFLAGS, (char *) &broad_req) != -1 &&
+		if (ioctl (socketFD, SIOCGIFFLAGS, (char *) &broad_req) != -1 &&
 		    (broad_req.ifr_flags & IFF_BROADCAST) &&
 		    (broad_req.ifr_flags & IFF_UP)
 		    )
 		{
 		    broad_req = *ifr;
-		    if (ifioctl (socketFD, SIOCGIFBRDADDR, &broad_req) != -1)
+		    if (ioctl (socketFD, SIOCGIFBRDADDR, &broad_req) != -1)
 			broad_addr = broad_req.ifr_addr;
 		    else
 			continue;
@@ -595,24 +545,35 @@ RegisterHostname (char *name)
 	    RegisterHostaddr ((struct sockaddr *)&in_addr, sizeof (in_addr),
 				QUERY);
 	}
-#if defined(IPv6) && defined(AF_INET6)
+#ifdef HAVE_GETADDRINFO
 	else {
 	    char sport[8];
 	    struct addrinfo *ai, *nai, hints;
 	    bzero(&hints,sizeof(hints));
+#  ifdef IPv6
+	    hints.ai_family = AF_UNSPEC;
+#  else
+	    hints.ai_family = AF_INET;
+#  endif
 	    hints.ai_socktype = SOCK_DGRAM;
 	    snprintf(sport, sizeof(sport), "%d", XDM_UDP_PORT);
 	    if (getaddrinfo(name, sport, &hints, &ai) == 0) {
 		for (nai = ai ; nai != NULL ; nai = nai->ai_next) {
-		    if ((nai->ai_family == AF_INET) ||
-		        (nai->ai_family == AF_INET6)) {
+		    if ((nai->ai_family == AF_INET)
+#ifdef IPv6
+                        || (nai->ai_family == AF_INET6)
+#endif
+                        ) {
 			if (((nai->ai_family == AF_INET) &&
 			  IN_MULTICAST(((struct sockaddr_in *) nai->ai_addr)
 			    ->sin_addr.s_addr))
+#ifdef IPv6
 			  || ((nai->ai_family == AF_INET6) &&
 			    IN6_IS_ADDR_MULTICAST(
 				&((struct sockaddr_in6 *) nai->ai_addr)
-				  ->sin6_addr)))
+				  ->sin6_addr))
+#endif
+                            )
 			{
 			    RegisterHostaddr(nai->ai_addr, nai->ai_addrlen,
 			      BROADCAST_QUERY);
@@ -625,12 +586,14 @@ RegisterHostname (char *name)
 		freeaddrinfo(ai);
 	    }
 	}
-#else
+#else /* !HAVE_GETADDRINFO */
 	/* Per RFC 1123, check first for IP address in dotted-decimal form */
 	else if ((in_addr.sin_addr.s_addr = inet_addr(name)) != -1)
 	    in_addr.sin_family = AF_INET;
 	else
 	{
+	    struct hostent	*hostent;
+
 	    hostent = gethostbyname (name);
 	    if (!hostent)
 		return;
@@ -645,7 +608,7 @@ RegisterHostname (char *name)
 # endif
 	RegisterHostaddr ((struct sockaddr *)&in_addr, sizeof (in_addr),
 			  QUERY);
-#endif /* IPv6 */
+#endif /* HAVE_GETADDRINFO */
     }
 }
 
@@ -675,7 +638,7 @@ InitXDMCP (char **argv)
     XdmcpWriteARRAYofARRAY8 (&directBuffer, &AuthenticationNames);
     if ((socketFD = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
 	return 0;
-#if defined(IPv6) && defined(AF_INET6)
+#ifdef IPv6
     socket6FD = socket (AF_INET6, SOCK_DGRAM, 0);
 #endif
 #ifdef SO_BROADCAST
@@ -686,7 +649,7 @@ InitXDMCP (char **argv)
 
     XtAddInput (socketFD, (XtPointer) XtInputReadMask, ReceivePacket,
 		(XtPointer) &socketFD);
-#if defined(IPv6) && defined(AF_INET6)
+#ifdef IPv6
     if (socket6FD != -1)
     XtAddInput (socket6FD, (XtPointer) XtInputReadMask, ReceivePacket,
 		(XtPointer) &socket6FD);
@@ -707,7 +670,7 @@ Choose (HostName *h)
     if (app_resources.xdmAddress)
     {
 	struct sockaddr_in  in_addr;
-#if defined(IPv6) && defined(AF_INET6)
+#ifdef IPv6
 	struct sockaddr_in6 in6_addr;
 #endif
 	struct sockaddr	*addr = NULL;
@@ -731,7 +694,7 @@ Choose (HostName *h)
 	    addr = (struct sockaddr *) &in_addr;
 	    len = sizeof (in_addr);
 	    break;
-#if defined(IPv6) && defined(AF_INET6)
+#ifdef IPv6
 	case AF_INET6:
 	    bzero(&in6_addr, sizeof(in6_addr));
 # ifdef SIN6_LEN
